<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:graphml="http://graphml.graphdrawing.org/xmlns" version="1.0">
    <xsl:output method="text" indent="yes"/>
    
    <xsl:preserve-space elements="graphml:edge"/>
    <xsl:strip-space elements="*"/>

    <xsl:template match="graphml:graphml">
        <xsl:text>digraph g {</xsl:text>
        <xsl:apply-templates/>
        <xsl:text>}</xsl:text>
    </xsl:template>
    
    <xsl:template match="graphml:graph">
        <xsl:if test="not(@id = 'base-graph')">
        <xsl:variable name="pos" select="count(preceding::graphml:graph)"/>
        <xsl:text>node [shape=box, style=filled, color="</xsl:text>
        <xsl:call-template name="select-color">
            <xsl:with-param name="pos" select="$pos"/>
        </xsl:call-template>
        <xsl:text>"];</xsl:text>
        </xsl:if>
        
        <xsl:apply-templates/>
    </xsl:template>
    
    <xsl:template match="graphml:node">
        <xsl:choose>
            <xsl:when test="parent::graphml:graph[@id = 'base-graph']">
                <xsl:apply-templates/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:text>"</xsl:text>
                <xsl:value-of select="@id"/>
                <xsl:text>"</xsl:text>
                <xsl:text> [shape=box];</xsl:text>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template match="graphml:edge">
        <xsl:text>"</xsl:text>
        <xsl:value-of select="@source"/>
        <xsl:text>"</xsl:text>
        
        <xsl:text> -&gt; </xsl:text>
        
        <xsl:text>"</xsl:text>
        <xsl:value-of select="@target"/>
        <xsl:text>"</xsl:text>
        <xsl:text>;</xsl:text>
    </xsl:template>
    
    
    <xsl:template name="select-color">
        <xsl:param name="pos"/>
        
        <xsl:choose>
            <xsl:when test="$pos = 0">
                <xsl:text>#90b0d4</xsl:text>
            </xsl:when>
            <xsl:when test="$pos = 1">
                <xsl:text>#e6f2ff</xsl:text>
            </xsl:when>
            <xsl:when test="$pos = 2">
                <xsl:text>#eee6ff</xsl:text>
            </xsl:when>
            <xsl:when test="$pos = 3">
                <xsl:text>#ffffe6</xsl:text>
            </xsl:when>
            <xsl:when test="$pos = 4">
                <xsl:text>#f9ecec</xsl:text>
            </xsl:when>
            <xsl:when test="$pos = 5">
                <xsl:text>#bfe0ba</xsl:text>
            </xsl:when>
            <xsl:when test="$pos = 6">
                <xsl:text>#bfd2e3</xsl:text>
            </xsl:when>
            <xsl:when test="$pos = 7">
                <xsl:text>#dbaddb</xsl:text>
            </xsl:when>
            <xsl:when test="$pos = 8">
                <xsl:text>#decbb2</xsl:text>
            </xsl:when>
            <xsl:when test="$pos = 9">
                <xsl:text>#e0b7b6</xsl:text>
            </xsl:when>
            <xsl:when test="$pos = 10">
                <xsl:text>#d9f09e</xsl:text>
            </xsl:when>
            <xsl:when test="$pos = 11">
                <xsl:text>#f5a1a4</xsl:text>
            </xsl:when>
            <xsl:when test="$pos = 12">
                <xsl:text>#faf0a5</xsl:text>
            </xsl:when>
            <xsl:when test="$pos = 13">
                <xsl:text>#8acca0</xsl:text>
            </xsl:when>
            <xsl:when test="$pos = 14">
                <xsl:text>#cfb88c</xsl:text>
            </xsl:when>
            <xsl:when test="$pos = 15">
                <xsl:text>#d18ecb</xsl:text>
            </xsl:when>
        </xsl:choose>
    </xsl:template>
</xsl:stylesheet>